#include "prefs.h"
#include <qfile.h>
#include <qtextstream.h>

const QString NeurPrefs::config = "/.xneur/xneurrc";

NeurPrefs::NeurPrefs() : mode("0"), point("55"), detect("0")
{
	load();
}

void NeurPrefs::load()
{
	if (getenv("HOME") == NULL)
		return;
	QFile file(getenv("HOME") + config);
	if (!file.open(IO_ReadOnly))
		return;
	QTextStream stream(&file);
	QString line;
	while (!stream.eof())
	{
		line = stream.readLine();
		int pos = line.find('#');
		if (pos >= 0)
			line = line.left(pos);
		line = line.stripWhiteSpace();
		if (line.isEmpty())
			continue;
		pos = line.find(' ');
		if (pos == -1)
			continue;
		QString param = line.left(pos).stripWhiteSpace();
		if (param == "DefaultMode")
		{
			mode = line.mid(pos).stripWhiteSpace();
			DBG1("mode set to '%s'", mode.latin1());
		}
		else if (param == "ConvPoint")
		{
			point = line.mid(pos).stripWhiteSpace();
			DBG1("point set to '%s'", point.latin1());
		}
		else if (param == "LangDetectMode")
		{
			detect = line.mid(pos).stripWhiteSpace();
			DBG1("detect set to '%s'", detect.latin1());
		}
		else if (param == "AddApp")
		{
			apps.append(line.mid(pos).stripWhiteSpace());
			DBG1("added app '%s'", line.mid(pos).stripWhiteSpace().latin1());
		}
	}
	file.close();
}

void NeurPrefs::save()
{
	if (getenv("HOME") == NULL)
		return;
	QFile file(getenv("HOME") + config);
	if (!file.open(IO_WriteOnly))
		return;
	QTextStream stream(&file);
	stream << "DefaultMode " << mode << '\n';
	DBG1("saved DefaultMode %s", mode.latin1());
	stream << "ConvPoint " << point << '\n';
	DBG1("saved ConvPoint %s", point.latin1());
	stream << "LangDetectMode " << detect << '\n';
	DBG1("saved LangDetectMode %s", detect.latin1());
	for (QStringList::const_iterator ci = apps.begin(); ci != apps.end(); ++ci)
	{
		stream << "AddApp " << *ci << '\n';
		DBG1("add app %s", (*ci).latin1());
	}
	file.close();
}
