#include "widget.h"
#include <dirent.h>
#include <signal.h>
#include <qapplication.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qfile.h>

NeurWidget::NeurWidget()
{
	QBoxLayout* layout = new QVBoxLayout(this, 10);
    QGridLayout* grid = new QGridLayout(layout, 0, 0, 10);
	QBoxLayout* appsLayout = new QHBoxLayout(layout, 10);
    QLabel* label;
    
	// mode list
	modeBox = new QComboBox(this);
	label = new QLabel(modeBox, tr("Default mode in window:"), this);
	grid->addWidget(label, 0, 0);
	grid->addWidget(modeBox, 0, 1);
	modeBox->insertItem(tr("Automatic"));
	modeBox->insertItem(tr("Manual"));
	modeBox->setCurrentItem(prefs.mode.toInt());

	// convertation point
	pointEdit = new QLineEdit(prefs.point, this);
	pointEdit->setValidator(new QIntValidator(pointEdit));
	label = new QLabel(pointEdit, tr("Convertation point:"), this);
	grid->addWidget(label, 2, 0);
	grid->addWidget(pointEdit, 2, 1);
	
	// language detect mode
	detectEdit = new QLineEdit(prefs.detect, this);
	detectEdit->setValidator(new QIntValidator(detectEdit));
	label = new QLabel(detectEdit, tr("Language detect mode:"), this);
	grid->addWidget(label, 3, 0);
	grid->addWidget(detectEdit, 3, 1);
	
	// apps list
	appsBox = new QListBox(this);
	appsBox->insertStringList(prefs.apps);
	appsBox->setSelected(0, true);
	label = new QLabel(appsBox, tr("Applications to process:"), this);
	grid->addWidget(label, 4, 0);
	appsLayout->addWidget(appsBox);

	// apps list control buttons
	addButton = new QPushButton(tr("Add..."), this);
	removeButton = new QPushButton(tr("Remove"), this);
	startButton = new QPushButton(tr("Start"), this);
	stopButton = new QPushButton(tr("Stop"), this);
	QBoxLayout* buttonsLayout = new QVBoxLayout(appsLayout, 10);
	buttonsLayout->addWidget(addButton);
	buttonsLayout->addWidget(removeButton);
	buttonsLayout->addWidget(startButton);
	buttonsLayout->addWidget(stopButton);
	connect(addButton, SIGNAL(clicked()), this, SLOT(addClicked()));
	connect(removeButton, SIGNAL(clicked()), this, SLOT(removeClicked()));
	connect(startButton, SIGNAL(clicked()), this, SLOT(startClicked()));
	connect(stopButton, SIGNAL(clicked()), this, SLOT(stopClicked()));
	
	// control buttons
	buttonsLayout = new QHBoxLayout(layout, 10);
	okButton = new QPushButton(tr("OK"), this);
	buttonsLayout->addWidget(okButton);
	applyButton = new QPushButton(tr("Apply"), this);
	buttonsLayout->addWidget(applyButton);
	quitButton = new QPushButton(tr("Exit"), this);
	buttonsLayout->addWidget(quitButton);
	connect(okButton, SIGNAL(clicked()), this, SLOT(okClicked()));
	connect(applyButton, SIGNAL(clicked()), this, SLOT(applyClicked()));
	connect(quitButton, SIGNAL(clicked()), qApp, SLOT(quit()));

	(getNeurPid() > 0 ? startButton : stopButton)->setDisabled(true);
	
	layout->activate();
}

void NeurWidget::okClicked()
{
    applyClicked();
    QApplication::exit(0);
}

void NeurWidget::applyClicked()
{
	prefs.mode = QString::number(modeBox->currentItem());
	prefs.point = pointEdit->text();
	prefs.detect = detectEdit->text();
	prefs.apps.clear();
	for (int i = 0; i < appsBox->numRows(); i++)
		prefs.apps.push_back(appsBox->text(i));
    prefs.save();

	if (stopButton->isEnabled())
	{
		stopClicked();
		startClicked();
	}
}

void NeurWidget::removeClicked()
{
	appsBox->removeItem(appsBox->currentItem());
}

void NeurWidget::addClicked()
{
	FILE* fp = popen("xprop WM_CLASS", "r");
	char buffer[NAME_MAX];
	const char* cap = "WM_CLASS(STRING)";

	if (fp == NULL)
		return;
	if (fgets(buffer, NAME_MAX, fp) == NULL)
		return;
	if (fclose(fp))
		return;
	if (strncmp(buffer, cap, strlen(cap)) != 0)
		return;
	char* p = strstr(buffer, "\", \"");
	if (p == NULL)
		return;
	if (strlen(p += 4) < 2)
		return;
	p[strlen(p) - 2] = '\0';
	appsBox->insertItem(p);
}

pid_t NeurWidget::getNeurPid() const
{
	/*
	DIR* proc = opendir("/proc");
	if (proc == NULL)
	{
		DBG0("failed to open /proc");
		return -1;
	}
	struct dirent* ent;
	while (ent = readdir(proc))
	{
		if (!isdigit(ent->d_name[0]))
			continue;
		char buffer[NAME_MAX] = "/proc/";
		strcat(buffer, ent->d_name);
		strcat(buffer, "/cmdline");
		FILE* cmd = fopen(buffer, "r");
		if (cmd == NULL)
		{
			DBG1("failed to open file '%s'", buffer);
			continue;
		}
		int rd = fread(buffer, 1, NAME_MAX, cmd);
		buffer[rd] = '\0';
		//DBG1("read '%s'", buffer);
		fclose(cmd);
		if (strstr(buffer, "xneur") == NULL)
			continue;
		DBG1("found xneur PID: %s", ent->d_name);
		// xneur PID found
		return atoi(ent->d_name);
	}
	closedir(proc);
	DBG0("xneur PID is not found");
	return 0;
	*/
	QFile file("/tmp/xneur.pid");
	if (!file.open(IO_ReadOnly))
		return -1;
	QTextStream stream(&file);
	QString line;
	if (!stream.eof())
	{
		pid_t xneurpid = stream.readLine().toInt();
		if (xneurpid <= 0)
		{
			DBG1("bad PID: %d", xneurpid);
			return -1;
		}
		DBG1("found xneur PID: %d", xneurpid);
		if (kill(xneurpid, 0))
		{
			DBG0("xneur is not responding");
			remove("/tmp/xneur.pid");
			return -1;
		}
		return xneurpid;
	}
	return 0;
}

void NeurWidget::startClicked()
{
	system("xneurwrp");
	if (getNeurPid() > 0)
	{
		startButton->setEnabled(false);
		stopButton->setEnabled(true);
	}
}

void NeurWidget::stopClicked()
{
	pid_t xneurpid = getNeurPid();
	if (xneurpid > 0)
		if (kill(xneurpid, SIGTERM) == -1)
			DBG1("failed to send SIGTERM to %d", xneurpid);
		else
		{
			startButton->setEnabled(true);
			stopButton->setEnabled(false);
		}
}
